package edu.chl.hajo.monopoly.view;

import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.event.IEventHandler;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import static java.lang.System.out;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JPanel;

/**

 @author hajo
 */
public class SpacePanel extends JPanel implements IEventHandler {

    // Name is unique id
    private final String name;
    private final List<Player> visitors = new ArrayList<>();
    private final Font font = new Font("SansSerif", Font.BOLD, 16);

    public SpacePanel(String name) {
        this.name = name;
        initGUI();
        initEvent();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2 = (Graphics2D) g;
        
        g2.drawRect(0, 0, getWidth(), getHeight());
        g2.drawString(name, 2, 10);

        int i = 1;
        for (Player visitor : visitors) {
            g2.drawImage(GUIResources.getPieceIcon(visitor.getName()),
                    i * 10, i * 10, null);
            i++;
            // out.println( "paint" + p.x + ":"  + p.y);
        }
    }

    @Override
    public void onEvent(Event evt) {
        if (evt.getTag() == Event.Tag.PLAYER_POSITION) {
            Player p = (Player) evt.getValue();
            if (p.getPosition().getName().equals(name)
                    && !visitors.contains(p)) {
                visitors.add(p);
                out.println("add" + p);
            } else {
                if (visitors.contains(p)) {
                    visitors.remove(p);
                    out.println("remove" + p);
                }
            }
        }
        // A Hack to make Swing update 
        Dimension size = this.getSize();
        this.setSize(size.height+1, size.width+1);
        validate();
        this.setSize(size);
       
        repaint();  // Must have
    }

    // Make background transparent
    private Color bg = new Color(0, 0, 0, 1);

    private void initGUI() {
        //this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        this.setBackground(bg);
    }

    private void initEvent() {
        EventBus.BUS.register(this);
    }
}
